<?php

/**
 * Spárování dopravců a platebních metod
 * @method string l(string $message) Localization
 * @property PcPayments2CarriersFree $module Odkaz na instanci modulu
 * @property array $errors
 * @property array $warnings
 * @property array $confirmations
 * @property array $informations
 */
class PcP2CFreeController extends ModuleAdminController {

    /** @var string Namespace pro Smarty */
    const NS = 'pc_p2c';

    /** @var Helper Odkaz na instanci helperu, který se právě používá rodič.kontrolerem. */
    protected $currentHelper;

    /** @var array Pro doplnění vlastních hodnot do this.fields_form */
    protected $fields_value_override = array();

    /** @var array Cache pro názvy platebních modulů */
    protected $paymentNamesCache = array();

    /** @var bool Je-li aktuální verze PS 1.6 */
    public $isVersion16;

    public function __construct() {
        $this->multishop_context = Shop::CONTEXT_ALL;

        parent::__construct();
        $this->meta_title = $this->module->getShortName();

        /* PS 1.6: */
        $this->isVersion16 = (version_compare(_PS_VERSION_, '1.6') >= 0);
        $this->bootstrap = true;
        if ($this->isVersion16) {
            $this->tpl_folder = 'xml_export16/';
        }
    }

    /**
     * @see AdminController::init()
     */
    public function init() {
        $this->loadMessages();
        parent::init();
        $this->initForm();
        $this->action = '';
        $this->display = 'list';/** NB: nesmí být "edit" ani "add", aby se nenačítal nedefinovaný ORM objekt. */
        if (Tools::isSubmit('submit' . get_class($this))) {
            $this->action = 'update';
        }

        /** Proměnná Smarty, kam si tento modul ukládá data (pseudo-namespace) */
        $this->context->smarty->assign(self::NS, array());
    }

    /**
     * Cache pro názvu platebních modulů
     * @param bool $update Pokud true, nastavení se aktualizuje podle instal. modulů.
     */
    protected function loadCache($update = false) {
        $cfgKey = PcPayments2CarriersFree::CFG_PREFIX . 'PaymentCache';
        $languageId = $this->context->language->id;

        if (empty($this->paymentNamesCache)) {
            if (Configuration::hasKey($cfgKey) && !$update) {
                $this->paymentNamesCache = (array) unserialize(Configuration::getGlobalValue($cfgKey));
            }
        }

        /* NB: Názvy se dají načíst jen pro aktuální jeden jazyk. */
        if ($update || !isset($this->paymentNamesCache[$languageId])) {
            /* Názvy platebních modulů se načtou pro všechny shopy */
            $sql = 'SELECT DISTINCT m.`id_module`, m.`name`
				FROM `' . _DB_PREFIX_ . 'module` m
				INNER JOIN `' . _DB_PREFIX_ . 'hook_module` hm ON hm.`id_module` = m.`id_module`
				INNER JOIN `' . _DB_PREFIX_ . 'hook` h ON hm.`id_hook` = h.`id_hook`
				WHERE h.`name` = "displayPayment"';
            $list = array();
            foreach (Db::getInstance()->executeS($sql) as $row) {
                /** @TBD include a skládání cesty je už v metodě getInstanceByName. Vyhodit? */
                $path = _PS_MODULE_DIR_ . '/' . $row['name'] . '/' . $row['name'] . '.php';
                if (file_exists($path)) {
                    require_once $path;
                    $module = get_object_vars(Module::getInstanceByName($row['name']));
                    $list[$row['name']] = $module['displayName'];
                }
            }
            $this->paymentNamesCache[$languageId] = $list;
            Configuration::updateGlobalValue($cfgKey, serialize($this->paymentNamesCache));
        }
    }

    /**
     * Načte z DB všechny hlášky, které se mají uživateli zobrazit.
     */
    protected function loadMessages() {
        static $done = false;
        if ($done) {
            return;
        }

        $key = PcPayments2CarriersFree::CFG_PREFIX . 'MSG' . $this->context->cookie->id_employee;
        if (Configuration::hasKey($key)) {
            $data = (array) unserialize(Configuration::getGlobalValue($key));
            foreach (array('errors', 'warnings', 'confirmations', 'informations') as $type) {
                if (isset($data[$type])) {
                    $this->$type = array_merge($this->$type, (array) $data[$type]);
                }
            }
            Configuration::updateGlobalValue($key, '');
        }
        $done = true;
    }

    /**
     * Uloží do DB všechny hlášky pro uživatele (před přesměrováním)
     */
    protected function saveMessages() {
        $key = PcPayments2CarriersFree::CFG_PREFIX . 'MSG' . $this->context->cookie->id_employee;
        $data = array();
        foreach (array('errors', 'warnings', 'confirmations', 'informations') as $type) {
            $data[$type] = $this->$type;
        }
        Configuration::updateGlobalValue($key, serialize($data));
    }

    /**
     * Před přesměrování uloží hlášky pro uživatele
     * @see AdminController::redirect()
     */
    protected function redirect() {
        $this->saveMessages();
        parent::redirect();
    }

    /**
     * Uloží odkaz na helper, se kterým se pak pracuje v renderX metodě rodiče.
     * (přes rodiče jinak není instance helperu dostupná)
     * @param Helper $helper
     */
    public function setHelperDisplay(Helper $helper) {
        $this->currentHelper = $helper;
        parent::setHelperDisplay($helper);
    }

    /**
     * Překrývá původní metodu, aby bylo možno doplnit vlastní pole do formuláře
     * 	(viz this.fields_value_override)
     * @param ModelObject objekt
     * @see AdminController::getFieldsValue()
     */
    public function getFieldsValue($obj) {
        parent::getFieldsValue($obj);
        $this->fields_value = array_merge($this->fields_value, $this->fields_value_override);
        return $this->fields_value;
    }

    /*     * ********  Zpracování požadavků ************* */

    /**
     * Uloží změněná data z formuláře
     * @see
     * @return bool
     */
    public function processUpdate() {
        /* stav modulu */
        $this->module->updateActive((int) Tools::getValue('active', 0));

        if (Tools::getValue('active', 0)) {
            $this->module->enable(true);
        } else {
            $this->module->disable(true);
        }

        /* povolené kombinace */
        $data = Tools::getValue('p2c', array());

        $deleteIds = '';
        $insertIds = '';
        foreach ($data as $key => $value) {
            if (preg_match('~^(\d+)_(\d+)$~', $key, $m)) {
                $deleteIds .= "($m[1], $m[2]), ";
                $insertIds .= "($m[1], $m[2]), ";
            }
        }
        $deleteIds = rtrim($deleteIds, ', ');
        $insertIds = rtrim($insertIds, ', ');

        if (empty($deleteIds)) {
            $sql = 'DELETE FROM `' . _DB_PREFIX_ . PcPayments2CarriersFree::DB_TABLE . '`'

            ;
            $result = Db::getInstance()->execute($sql);
        } else {
            $sql1 = 'DELETE FROM `' . _DB_PREFIX_ . PcPayments2CarriersFree::DB_TABLE . '`
				WHERE (`id_carrier`, `id_payment`) NOT IN (' . $deleteIds . ')'

            ;
            $sql2 = 'INSERT IGNORE INTO `' . _DB_PREFIX_ . PcPayments2CarriersFree::DB_TABLE . '`
				(`id_carrier`, `id_payment`  )
				VALUES ' . $insertIds;

            $result = Db::getInstance()->execute($sql1) && Db::getInstance()->execute($sql2);
        }
        CacheCore::clean(PcPayments2CarriersFree::CFG_PREFIX . '*');
        if ($result) {
            $this->confirmations[] = $this->l('Setting has been successfully updated');
        } else {
            $this->errors[] = $this->l('Failed to save setting');
        }
        $this->redirect_after = self::$currentIndex . '&token=' . $this->token;
    }

    /*     * ********  Zobrazení   ************** */

    /**
     * Připraví proměnné do formuláře
     */
    protected function initForm() {
        $formLegend = '';

        if (Shop::getContext() !== Shop::CONTEXT_SHOP && count(Shop::getContextListShopID()) > 1) {
            $formLegend .= $this->l('Free module version works only with default e-shop (ID = 1). For full Multistore compatibility, you must upgrade to Pro version.');
        }

        $fields = array(
            'legend' => array(
                'title' => $this->l('Setting'),
                'image' => $this->module->getPathUri() . 'img/wrench.png',
            ),
            'input' => array(
                array(
                    'name' => 'active',
                    'label' => $this->l('Enable module'),
                    'type' => ($this->isVersion16 ? 'switch' : 'radio'),
                    'default' => 1, /* NB: všechny moduly jsou po instalaci automaticky povoleny. */
                    'class' => 't',
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'enable',
                            'value' => 1,
                            'label' => $this->l('Yes')
                        ),
                        array(
                            'id' => 'disable',
                            'value' => 0,
                            'label' => $this->l('No')
                        )
                    )
                ),
                array(
                    'label' => $this->l('Assign payment methods to each carrier in the table'),
                    'name' => 'grid',
                    'type' => 'free',
                    'desc' => $formLegend,
                ),
            ),
            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'button',
            )
        );
        if ($this->isVersion16) {
            $fields['legend']['icon'] = 'icon-cogs';
            unset($fields['legend']['image']);
            $fields['buttons'] = array(
                'save' => array(
                    'name' => 'submit' . get_class($this),
                    'title' => $this->l('Save'),
                    'type' => 'submit',
                    'icon' => 'process-icon-save',
                    'class' => 'pull-right',
                ),
            );
            unset($fields['submit']);
        }

        $this->fields_form = array(0 => array('form' => $fields));
    }

    /**
     * Zobrazí formulář, aniž by se použila akce add/edit.
     */
    public function renderList() {
        return $this->renderForm();
    }

    /**
     * Vytvoří HTML s nastavením modulu.
     * @see AdminController::renderForm()
     * @return string
     */
    public function renderForm() {
        $this->show_form_cancel_button = false;

        $helper = $this->getHelperForm();
        $helper->fields_value['grid'] = $this->getFormGrid();
        $helper->fields_value['active'] = (int) $this->module->isActive();
        $output = $helper->generateForm($this->fields_form);
        return $output;
    }

    /**
     * Vytvoří a nastaví helperForm.
     * @return HelperForm
     */
    protected function getHelperForm() {
        $helper = new HelperForm;

        $helper->module = $this->module;
        $helper->name_controller = $this->module->name;
        $helper->token = $this->token;
        $helper->currentIndex = self::$currentIndex . '&token=' . $this->token;
        $helper->title = $this->module->displayName;
        $default_lang = (int) Configuration::get('PS_LANG_DEFAULT');
        $helper->default_form_language = $default_lang;
        /* 		$helper->allow_employee_form_lang = false; -- TBD: co za hodnotu to chce? */
        $helper->show_toolbar = true;
        $helper->toolbar_scroll = false;
        $helper->submit_action = 'submit' . get_class($this);

        /** @FIXME tady by se měly std. kopírovat buttony z this.toolbar_btn (aspon teda v 1.5) ! */
        $helper->toolbar_btn = array(
            'save' => array(
                'desc' => $this->l('Save'),
                'href' => self::$currentIndex . '&token=' . $this->token . '&submit' . get_class($this), /** @XXX ok takhle? */
            ),
            'back' => array(
                'href' => self::$currentIndex . '&token=' . $this->token,
                'desc' => $this->l('Back to module list'),
            )
        );

        return $helper;
    }

    /**
     * Vrací vygenerovanou tabulku pro formulář
     * @return string HTML
     */
    protected function getFormGrid() {
        $this->loadCache();


        $sql = 'SELECT car.*, pay.*, '
                . ' COUNT(p2c.`id_carrier`) AS active '
                . ' FROM
				(SELECT DISTINCT c.`id_carrier`, c.`name` AS carrier_name, cl.`delay` AS `carrier_description`
				FROM `' . _DB_PREFIX_ . 'carrier` c '
                . ' INNER JOIN `' . _DB_PREFIX_ . 'carrier_lang` cl ON cl.`id_carrier` = c.`id_carrier`'
                . ' AND cl.`id_shop` = ' . $this->context->shop->id
                . ' AND cl.`id_lang` = ' . $this->context->language->id . '
				WHERE c.`active` = 1 AND c.`deleted` = 0) car
			CROSS JOIN
				(SELECT DISTINCT m.`id_module`, h.`id_hook`, m.`name` AS payment_module, hm.`position`
				FROM `' . _DB_PREFIX_ . 'module` m '
                . ' LEFT JOIN `' . _DB_PREFIX_ . 'hook_module` hm ON hm.`id_module` = m.`id_module`'
                . ' AND hm.`id_shop` = ' . $this->context->shop->id
                . ' LEFT JOIN `' . _DB_PREFIX_ . 'hook` h ON hm.`id_hook` = h.`id_hook`
				WHERE h.`name` = "displayPayment") pay
			LEFT JOIN `' . _DB_PREFIX_ . PcPayments2CarriersFree::DB_TABLE . '` p2c ON car.`id_carrier` = p2c.`id_carrier`
				AND pay.`id_module` = p2c.`id_payment`'
                . ' GROUP BY car.`id_carrier`, pay.`id_module`
		';

        $data = array();
        $header = array();
        $languageId = $this->context->language->id;
        foreach (Db::getInstance()->executeS($sql) as $row) {
            if (!isset($data[$row['id_carrier']])) {
                $data[$row['id_carrier']] = array(
                    'name' => $row['carrier_name'],
                    'description' => $row['carrier_description'],
                    'childNodes' => array(),
                );
            }

            $data[$row['id_carrier']]['childNodes'][] = $row;

            /** @TBD co když modul vůbec nemá logo? Nechat PS, aby použil automatický obrázek, nebo definovat tady? */
            $imgType = (file_exists(_PS_MODULE_DIR_ . $row['payment_module'] . '/logo.gif') ? 'logo.gif' : 'logo.png');
            if (!isset($this->paymentNamesCache[$languageId][$row['payment_module']])) {
                $this->loadCache(true);
            }
            /* NB: URL obrázku je přesně podle AdminPaymentCtrl */
            $header[$row['id_module']] = array(
                'module' => $row['payment_module'],
                'src' => __PS_BASE_URI__ . 'modules/' . $row['payment_module'] . '/' . $imgType,
                'name' => (isset($this->paymentNamesCache[$languageId][$row['payment_module']]) ? $this->paymentNamesCache[$languageId][$row['payment_module']] : $row['payment_module']),
            );
        }

        $this->context->smarty->assign(self::NS, array(
            'data' => $data,
            'header' => $header,
            'isVersion16' => $this->isVersion16,
        ));
        return $this->module->display($this->module->getLocalPath() . $this->module->name . '.php', 'form.tpl');
    }

}
