<?php

/**
 * Override pro modul pcpayments2carriers.
 * Kvůli Autoloaderu musí končit koncovým tagem PHP.
 */
class Module extends ModuleCore {

    /**
     * Pokud není daná platební metoda povolená pro vybraného dopravce, vrací pole s názvem aktuálního kontroleru.
     * @param int $hookID
     * @param bool $dispatch
     * @see ModuleCore::getExceptions()
     * @return array
     */
    public static function getExceptionsStatic($id_module, $id_hook, $dispatch = false) {

        $controller = Dispatcher::getInstance()->getController();

        /**
         *  Is it payment module running from order ? 
         */
        if (!preg_match('/^(order|orderopc)$/i', $controller)) {
            return parent::getExceptionsStatic($id_module, $id_hook, $dispatch);
        }

        $displayPaymentId = Hook::getIdByName("displayPayment");

        /**
         *  Is it payment Hook ?
         */
        if ($id_hook != $displayPaymentId) { /* NB: přibližné porovnání je tu úmyslně */
            return parent::getExceptionsStatic($id_module, $id_hook, $dispatch);
        }

        $module = self::getInstanceByName('pcpayments2carriersfree');

        /**
         *  Is module active in this shop instance ? 
         */
        if (!$module->isActive()) {
            return parent::getExceptionsStatic($id_module, $id_hook, $dispatch);
        }

        /**
         * For unidentified carriers show all payments methods. 
         */
        if (empty(Context::getContext()->cart->id_carrier)) {
            return parent::getExceptionsStatic($id_module, $id_hook, $dispatch);
        }

        $payments_ids = $module->getPaymentsByCarrierId(Context::getContext()->cart->id_carrier);

        /**
         * Unallowed payments methods will be hidden.
         */
        if (in_array($id_module, $payments_ids)) {
            return parent::getExceptionsStatic($id_module, $id_hook, $dispatch);
        } else {
            return array($controller);
        }
    }

}

?>
