<?php

/**
 * Spárování plateb a dopravců.
 * Všechna nastavení, která tento modul používá, musí začínat řetězcem module::CFG_PREFIX.
 *
 * @property Context $context Globální registr
 * @method string l() Překlady
 */
if (!defined('_PS_VERSION_'))
    exit;

class PcPayments2CarriersFree extends Module {

    /** @var string Prefix pro nastavení, ukládaná do DB. (zbývá max. 21 znaků do názvu) */
    const CFG_PREFIX = 'PC_P2CFREE_';

    /** @var string Název DB tabulky */
    const DB_TABLE = 'pc_p2cfree';

    /** @var string Zkrácený název modulu pro menu BO (max. 32 znaků) */
    protected $shortName = '';

    public function __construct() {
        $this->name = 'pcpayments2carriersfree';
        $this->controllerClass = 'PcP2CFree'; /* "Controller" se doplní automaticky */
        $this->tab = 'payments';
        $this->version = '0.9.2';

        $this->author = 'PrestaCenter';
        $this->need_instance = 1;

        parent::__construct();

        $this->displayName = $this->l('PrestaCenter Payments 2 Carriers Free');

        $this->shortName = $this->l('Payments 2 Carriers');
        $this->description = $this->l('Assign payments to carriers');
    }

    /**
     * Instalátor.
     * @return boolean
     */
    public function install() {
        try {


            $this->installDbData();

            /* Instaluje modul do DB a nakopíruje potřebné skripty */
            if (method_exists($this, 'updateTranslationsAfterInstall'))
                $this->updateTranslationsAfterInstall(false);
            /* NB: Při instalaci je modul automaticky aktivován pro všechny shopy v kontextu. */
            if (!parent::install())
                throw new RuntimeException; /* popis chyby je už uložený v this->_errors[] */

            /* Přidá odkaz na modul do menu administrace */
            $this->addTab();
            $this->registerHook('actionCarrierUpdate');

            return true;
        } catch (RuntimeException $e) {
            $this->_errors[] = $e->getMessage();
            return false;
        }
    }

    /**
     * Odinstaluje modul
     * @return boolean
     */
    public function uninstall() {
        try {

            /* Odinstaluje modul z DB a odstraní všechny jeho skripty z /override i hooky */
            if (!parent::uninstall())
                throw new RuntimeException; /* popis chyby je už uložený v this->_errors[] */

            $this->removeTab();
            $this->uninstallDbData();
            return true;
        } catch (RuntimeException $e) {
            $this->_errors[] = $e->getMessage();
            return false;
        }
    }

    /**
     * Nainstaluje override třídy
     * @throws RuntimeException
     * @see ModuleCore::installOverrides()
     */
    public function installOverrides() {
        $errorFlag = false;
        $from = $this->getLocalPath() . 'override' . DS;
        $to = rtrim(_PS_ROOT_DIR_, DS) . DS . 'override' . DS;

        /** @TODO Přidat ověření, že složky existují a cílová je zapisovatelná */
        foreach (Tools::scandir($this->getLocalPath() . 'override', 'php', '', true) as $file) {
            if (!file_exists($to . dirname($file)) && !mkdir($to . dirname($file)))
                $errorFlag = true;
            if (!copy($from . $file, $to . $file))
                $errorFlag = true;
        }
        Autoload::getInstance()->generateIndex();

        if ($errorFlag)
            throw new RuntimeException($this->l('Failed to copy module files'));
        return true;
    }

    /**
     * Vytvoří v DB tabulky a vloží výchozí data
     * @throws RuntimeException
     */
    protected function installDbData() {
        $sql1 = 'CREATE TABLE `' . _DB_PREFIX_ . self::DB_TABLE . '` (
            `id_carrier` INT(11) NOT NULL,
            `id_payment` INT(11) NOT NULL,
			PRIMARY KEY (`id_carrier`, `id_payment`)
            ) ENGINE = ' . _MYSQL_ENGINE_ . ' DEFAULT CHARSET=utf8';


        /* Výchozí data - povolení všech platebních modulů pro všechny dopravce. */
        $sql2 = 'INSERT INTO `' . _DB_PREFIX_ . self::DB_TABLE . '` (`id_carrier`, `id_payment`)
			SELECT DISTINCT car.`id_carrier`, pay.`id_module`
			FROM
				(SELECT c.`id_carrier`
				FROM `' . _DB_PREFIX_ . 'carrier` c
				WHERE c.`deleted` = 0) car
			CROSS JOIN
				(SELECT m.`id_module`
				FROM `' . _DB_PREFIX_ . 'module` m
				LEFT JOIN `' . _DB_PREFIX_ . 'hook_module` hm ON hm.`id_module` = m.`id_module`
				LEFT JOIN `' . _DB_PREFIX_ . 'hook` h ON hm.`id_hook` = h.`id_hook`
				WHERE h.`name` = "displayPayment") pay';


        if (!Db::getInstance()->execute($sql1) || !Db::getInstance()->execute($sql2))
            throw new RuntimeException($this->l('Failed to install database tables of this module'));
    }

    /**
     * Odstraní DB tabulky modulu
     * @throws RuntimeException
     */
    protected function uninstallDbData() {
        if (!Db::getInstance()->execute('DROP TABLE `' . _DB_PREFIX_ . self::DB_TABLE . '`') || !Db::getInstance()->execute('DELETE FROM `' . _DB_PREFIX_ . 'configuration`
				WHERE `name` LIKE "' . self::CFG_PREFIX . '%"'))
            throw new RuntimeException($this->l('Failed to delete the data from the database of this module') . ' : ' . $db->getMsgError());
    }

    /**
     * Přidá odkaz na modul do menu administrace (první nabídka zleva, poslední volba)
     * @throws RuntimeException
     */
    protected function addTab() {
        $id_parent = Tab::getIdFromClassName('AdminParentModules');
        if (!$id_parent)
            throw new RuntimeException($this->l('Unable to add the module to the administration menu') . ' : ' . Db::getInstance()->getMsgError());

        /** @TODO kontrola délky názvu, aby this.shortName nebylo delší než 32 znaků, případně ořez. */
        /* Název tabu je pro všechny nainstal.jazyky stejný, uživatel si upraví. */
        $tabNames = array();
        foreach (Language::getLanguages(false) as $lang)
            $tabNames[$lang['id_lang']] = $this->shortName;

        $tab = new Tab(); /* null, $this->context->language->id); */
        $tab->class_name = $this->controllerClass;
        $tab->name = $tabNames;
        $tab->module = $this->name;
        $tab->id_parent = $id_parent;

        if (!$tab->save())
            throw new RuntimeException($this->l('Unable to add the module to the administration menu'));
    }

    /**
     * Odstraní odkaz na tento modul z menu administrace
     * @throws RuntimeException
     */
    protected function removeTab() {
        if (!Tab::getInstanceFromClassName($this->controllerClass)->delete())
            throw new RuntimeException($this->l('Failed to remove the module from the main BO menu'));
    }

    /**
     * Handler pro stránku s nastavením modulu.
     * Přesměruje na hlavní stránku modulu.
     */
    public function getContent() {
        $id = Tab::getIdFromClassName($this->controllerClass);
        $token = Tools::getAdminToken($this->controllerClass . $id . (int) $this->context->employee->id);
        header('Location: index.php?controller=' . $this->controllerClass . '&token=' . $token);
        exit;
    }

    /*     * ***************** Zpracování akcí ************* */

    /** @XXX Prozatím ponecháno kvůli možnosti vypnutí modulu přes seznam modulů */

    /**
     * Activate current module.
     * @param bool $forceAll If true, enable module for all shops
     * @see ModuleCore::enable()
     */
    public function enable($forceAll = false) {
        parent::enable($forceAll);
        $this->updateActive(1);
    }

    /**
     * Deactivate current module.
     * @param bool $forceAll If true, disable module for all shops
     * @see ModuleCore::disable()
     */
    public function disable($forceAll = false) {
        parent::disable($forceAll);
        $this->updateActive(0);
    }

    /*     * ************  Hooky ************************* */

    /**
     * Update při změně ID dodavatele
     */
    public function hookActionCarrierUpdate($params) {
        if (!empty($params['id_carrier']) && isset($params['carrier']) && is_object($params['carrier'])) {
            $sql = 'UPDATE `' . _DB_PREFIX_ . self::DB_TABLE . '` SET `id_carrier` = ' . $params['carrier']->id . '
				WHERE `id_carrier` = ' . $params['id_carrier'];
            Db::getInstance()->execute($sql);
            CacheCore::clean(self::CFG_PREFIX . '*');
        }
    }

    public function getPaymentsByCarrierId($carrier_id) {
        $cache_id = self::CFG_PREFIX . 'payments_' . $carrier_id;
        $payments_ids = array();

        if (!Cache::isStored($cache_id)) {
            $sql = 'SELECT `id_payment` ' .
                    'FROM `' . _DB_PREFIX_ . PcPayments2CarriersFree::DB_TABLE . '` ' .
                    'WHERE `id_carrier` = ' . (int) $carrier_id;

            foreach (Db::getInstance()->executeS($sql) as $row) {
                $payments_ids[] = $row['id_payment'];
            }
            Cache::store($cache_id, $payments_ids);
        } else {
            $payments_ids = Cache::retrieve($cache_id);
        }
        return $payments_ids;
    }

    /*     * **********   Pomocné metody ***************** */

    /**
     * Updatuje nastavení, pro které shopy je modul aktivní.
     * @param int $status Aktivní nebo ne (1|0)
     */
    public function updateActive($status) {
        Configuration::updateGlobalValue(self::CFG_PREFIX . 'Active', (int) $status);
    }

    /**
     * Zjistí, jestli je modul aktivní pro aktuálně nastavený shop.
     * (Necachuje data, čte pokaždé aktuální data z DB.)
     * @return bool
     */
    public function isActive() {
        if (Configuration::hasKey(self::CFG_PREFIX . 'Active')) {
            $status = $this->context->shop->id == 1 && (bool) Configuration::getGlobalValue(self::CFG_PREFIX . 'Active');
        } else {
            $status = (bool) Module::isEnabled($this->name);
            $this->updateActive((int) $status);
        }
        return $status;
    }

    /**
     * Vrací zkrácený název modulu
     * @return string
     */
    public function getShortName() {
        return $this->shortName;
    }

}
